//+------------------------------------------------------------------+
//|                                           Schaff Trend Cycle.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  LimeGreen
#property indicator_color2  DarkOrange
#property indicator_color3  DarkOrange
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2
#property indicator_minimum  -5
#property indicator_maximum 105

//
//
//
//
//

extern string TimeFrame   = "Current time frame";
extern int    STCPeriod   = 15;
extern int    TrixPeriod  = 30;
extern int    Price       = PRICE_MEDIAN;
extern bool   Interpolate = true; 

//
//
//
//
//

double stcBuffer[];
double stcBufferDa[];
double stcBufferDb[];
double valBuffer[];
double fastKBuffer[];
double fastDBuffer[];
double fastKKBuffer[];
double slope[];

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(8);
   SetIndexBuffer(0,stcBuffer);
   SetIndexBuffer(1,stcBufferDa);
   SetIndexBuffer(2,stcBufferDb);
   SetIndexBuffer(3,valBuffer);
   SetIndexBuffer(4,fastKBuffer);
   SetIndexBuffer(5,fastDBuffer);
   SetIndexBuffer(6,fastKKBuffer);
   SetIndexBuffer(7,slope);
   
   //
   //
   //
   //
   //
       
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame == "returnBars";     if (returnBars)     return(0);
      calculateValue    = TimeFrame == "calculateValue"; if (calculateValue) return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);
          
   //
   //
   //
   //
   //
   
   
   IndicatorShortName(timeFrameToString(timeFrame)+" Schaff Trix Trend Cycle ("+STCPeriod+","+TrixPeriod+")");
   return(0);
}

int deinit()
{
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double ema3[];
int start()
{
   int i,r,counted_bars=IndicatorCounted();
      if(counted_bars < 0) return(-1);
      if(counted_bars > 0) counted_bars--;
           int limit = MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { stcBuffer[0] = limit+1; return(0); }

   //
   //
   //
   //
   //
   
   if (calculateValue || timeFrame == Period())
   {
      if (ArraySize(ema3)!=Bars) ArrayResize(ema3,Bars);
      if (slope[limit] == -1) ClearPoint(limit,stcBufferDa,stcBufferDb);
      for(i = limit, r = Bars-i-1; i >= 0; i--,r++)
      {
         double price   = MathLog(iMA(NULL,0,1,0,MODE_SMA,Price,i));
         double ema1    = iEma(price,TrixPeriod,i,0);
         double ema2    = iEma(ema1 ,TrixPeriod,i,1);
                ema3[r] = iEma(ema2 ,TrixPeriod,i,2);
         valBuffer[i]   = (ema3[r]-ema3[r-1])*10000;

         //
         //
         //
         //
         //
         
         double lowVal  = minValue(valBuffer,i);
         double highVal = maxValue(valBuffer,i)-lowVal;
            if (highVal > 0)
                  fastKBuffer[i] = 100*((valBuffer[i]-lowVal)/highVal);
            else  fastKBuffer[i] = fastKBuffer[i+1];
                  fastDBuffer[i] = fastDBuffer[i+1]+0.5*(fastKBuffer[i]-fastDBuffer[i+1]);
               
         //
         //
         //
         //
         //
                     
         double lowStoch  = minValue(fastDBuffer,i);
         double highStoch = maxValue(fastDBuffer,i)-lowStoch;
            if (highStoch > 0)
                  fastKKBuffer[i] = 100*((fastDBuffer[i]-lowStoch)/highStoch);
            else  fastKKBuffer[i] = fastKKBuffer[i+1];
                  stcBuffer[i]    = stcBuffer[i+1]+0.5*(fastKKBuffer[i]-stcBuffer[i+1]);
                  stcBufferDa[i]  = EMPTY_VALUE;
                  stcBufferDb[i]  = EMPTY_VALUE;
                  slope[i]        = slope[i+1];
                     if (stcBuffer[i]>stcBuffer[i+1]) slope[i] =  1;
                     if (stcBuffer[i]<stcBuffer[i+1]) slope[i] = -1;
                     if (slope[i] == -1) PlotPoint(i,stcBufferDa,stcBufferDb,stcBuffer);
      }   
      return(0);
   }
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (i=limit; i>=0; i--)
   {
       int y = iBarShift(NULL,timeFrame,Time[i]);               
          stcBuffer[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",STCPeriod,TrixPeriod,Price,0,y);
          slope[i]       = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",STCPeriod,TrixPeriod,Price,7,y);
          stcBufferDa[i] = EMPTY_VALUE;
          stcBufferDb[i] = EMPTY_VALUE;
            
          //
          //
          //
          //
          //
       
          if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
          if (!Interpolate) continue;

          //
          //
          //
          //
          //

          datetime time = iTime(NULL,timeFrame,y);
             for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
             for(int x = 1; x < n; x++)
                stcBuffer[i+x] = stcBuffer[i] + (stcBuffer[i+n] - stcBuffer[i]) * x/n;
   }
   for (i=limit; i>=0; i--) if (slope[i] == -1) PlotPoint(i,stcBufferDa,stcBufferDb,stcBuffer);
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workEma[][3];
double iEma(double price, double period, int r, int instanceNo=0)
{
   if (ArrayRange(workEma,0)!= Bars) ArrayResize(workEma,Bars); r = Bars-r-1;

   //
   //
   //
   //
   //
      
   double alpha = 2.0 / (1.0+period);
          workEma[r][instanceNo] = workEma[r-1][instanceNo]+alpha*(price-workEma[r-1][instanceNo]);
   return(workEma[r][instanceNo]);
} 

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double minValue(double& array[],int shift)
{
   double minValue = array[shift];
            for (int i=1; i<STCPeriod; i++) minValue = MathMin(minValue,array[shift+i]);
   return(minValue);
}
double maxValue(double& array[],int shift)
{
   double maxValue = array[shift];
            for (int i=1; i<STCPeriod; i++) maxValue = MathMax(maxValue,array[shift+i]);
   return(maxValue);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void ClearPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
   {
      if (first[i+2] == EMPTY_VALUE) 
      {
          first[i]    = from[i];
          first[i+1]  = from[i+1];
          second[i]   = EMPTY_VALUE;
      }
      else
      {
          second[i]   = from[i];
          second[i+1] = from[i+1];
          first[i]    = EMPTY_VALUE;
      }
   }
   else
   {
         first[i]   = from[i];
         second[i]  = EMPTY_VALUE;
      }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   StringToUpper(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}